@@ -30,6 +30,7 @@ import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.ScheduledCommand;
 import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
+import com.google.gwt.safehtml.shared.SafeHtmlUtils;
 import com.google.gwt.user.client.rpc.AsyncCallback;
 import com.google.web.bindery.event.shared.EventBus;
 import com.sencha.gxt.core.client.IdentityValueProvider;
@@ -115,10 +116,11 @@ public class HistoryPanel extends FramedPanel implements MonitorDialogListener {
 					@Override
 					public void render(Context context, OpHistory value,
 							SafeHtmlBuilder sb) {
-						sb.appendHtmlConstant("<p style='margin: 5px 5px 10px'><b>Date:</b>").appendEscaped(
-								value.getDate()).appendHtmlConstant("</p>");
-						sb.appendHtmlConstant("<p style='margin: 5px 5px 10px'><b>Description:</b>")
-								.appendEscaped(value.getDescription()).appendHtmlConstant("</p>");
+						sb.appendHtmlConstant("<p style='margin: 5px 5px 10px'><b>Date:</b>"
+								+ SafeHtmlUtils.htmlEscape(value.getDate()) + "</p>");
+						sb.appendHtmlConstant("<p style='margin: 5px 5px 10px'><b>Description:</b>"
+								+ SafeHtmlUtils.htmlEscape(value
+										.getDescription()) + "</p>");
 					}
 				});
 
@@ -129,8 +131,9 @@ public class HistoryPanel extends FramedPanel implements MonitorDialogListener {
 
 			@Override
 			public void render(Context context, String value, SafeHtmlBuilder sb) {
-				sb.appendHtmlConstant("<span title='").appendEscaped(value).appendHtmlConstant("'>").appendEscaped(value).
-						appendHtmlConstant("</span>");
+				sb.appendHtmlConstant("<span title='"
+						+ SafeHtmlUtils.htmlEscape(value) + "'>"
+						+ SafeHtmlUtils.htmlEscape(value) + "</span>");
 
 			}
 
