@@ -8,6 +8,8 @@ import org.gcube.portlets.user.td.columnwidget.client.dimension.DimensionRowSele
 import org.gcube.portlets.user.td.columnwidget.client.dimension.DimensionRowSelectionListener;
 import org.gcube.portlets.user.td.columnwidget.client.dimension.DimensionRowsProperties;
 import org.gcube.portlets.user.td.gwtservice.client.rpc.TDGWTServiceAsync;
+import org.gcube.portlets.user.td.gwtservice.shared.exception.TDGWTIsFinalException;
+import org.gcube.portlets.user.td.gwtservice.shared.exception.TDGWTIsLockedException;
 import org.gcube.portlets.user.td.gwtservice.shared.exception.TDGWTSessionExpiredException;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.ColumnData;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.DimensionRow;
@@ -49,6 +51,7 @@ import com.sencha.gxt.widget.core.client.container.BoxLayoutContainer.BoxLayoutD
 import com.sencha.gxt.widget.core.client.container.BoxLayoutContainer.BoxLayoutPack;
 import com.sencha.gxt.widget.core.client.container.HBoxLayoutContainer;
 import com.sencha.gxt.widget.core.client.container.HBoxLayoutContainer.HBoxLayoutAlign;
+import com.sencha.gxt.widget.core.client.container.MarginData;
 import com.sencha.gxt.widget.core.client.container.SimpleContainer;
 import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer;
 import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer.VerticalLayoutData;
@@ -59,31 +62,39 @@ import com.sencha.gxt.widget.core.client.event.TriggerClickEvent.TriggerClickHan
 import com.sencha.gxt.widget.core.client.form.ComboBox;
 import com.sencha.gxt.widget.core.client.form.DateField;
 import com.sencha.gxt.widget.core.client.form.FieldLabel;
+import com.sencha.gxt.widget.core.client.form.FieldSet;
 import com.sencha.gxt.widget.core.client.form.Radio;
 import com.sencha.gxt.widget.core.client.form.TextField;
+import com.sencha.gxt.widget.core.client.form.validator.RegExValidator;
 
 /**
  * 
  * @author "Giancarlo Panichi"
  * 
  */
-public class EditRowPanel extends FramedPanel 
- implements MonitorDialogListener {
-	protected String WIDTH = "510px";
-	protected String HEIGHT = "370px";
-	protected String FIELDSHEIGHT = "370px";
-	protected int LABELSIZE = 140;
+public class EditRowPanel extends FramedPanel implements MonitorDialogListener {
+	private static final String ITEM_CREATE_ROW = "NewRow";
 
-	protected EditRowDialog parent;
-	protected TRId trId;
-	protected EventBus eventBus;
-	protected ArrayList<ColumnData> columns;
-	protected RowRaw rowRaw;
-	protected boolean editRow;
+	private static final String GEOMETRY_REGEXPR = "(\\s*POINT\\s*\\(\\s*(-)?\\d+(\\.\\d+)?\\s+(-)?\\d+(\\.\\d+)?\\s*\\)\\s*$)"
+			+ "|(\\s*LINESTRING\\s*\\((\\s*(-)?\\d+(\\.\\d+)?\\s+(-)?\\d+(\\.\\d+)?\\s*,)+\\s*((-)?\\d+(\\.\\d+)?\\s+(-)?\\d+(\\.\\d+)?\\s*)\\)\\s*$)";
 
-	
+	private static final String WIDTH = "600px";
+	private static final String HEIGHT = "370px";
+	private static final String CONTAINERHEIGHT = "370px";
+	private static final int LABELSIZE = 120;
+	private static final String FIELDSETWIDTH = "640px";
+
+	private EditRowDialog parent;
+	private TRId trId;
+	private EventBus eventBus;
+	private ArrayList<ColumnData> columns;
+	private ArrayList<RowRaw> rowsRaw;
+	private boolean editRow;
+
+	private VerticalLayoutContainer v;
+
+	private ArrayList<String> rowsId;
 
-	protected VerticalLayoutContainer v;
 	private TextButton btnSave;
 	private TextButton btnClose;
 
@@ -94,20 +105,20 @@ public class EditRowPanel extends FramedPanel
 	 * 
 	 * @param parent
 	 * @param trId
-	 * @param rowRaw
+	 * @param rowsRaw
 	 * @param eventBus
 	 */
-	public EditRowPanel(EditRowDialog parent, TRId trId, RowRaw rowRaw,
-			EventBus eventBus) {
+	public EditRowPanel(EditRowDialog parent, TRId trId,
+			ArrayList<RowRaw> rowsRaw, EventBus eventBus) {
 		super();
 		this.parent = parent;
 		this.trId = trId;
-		this.rowRaw = rowRaw;
+		this.rowsRaw = rowsRaw;
 		this.eventBus = eventBus;
 		this.editRow = true;
-		Log.debug("Create EditRowPanel(): [" + trId.toString() + " , RowRaw:"
-				+ rowRaw + "]");
-		if (rowRaw == null) {
+		Log.debug("Create EditRowPanel(): [" + trId.toString() + " , RowsRaw:"
+				+ rowsRaw + "]");
+		if (rowsRaw == null || rowsRaw.isEmpty()) {
 			UtilsGXT3.alert("Attentions", "No row selected");
 		} else {
 			init();
@@ -144,11 +155,11 @@ public class EditRowPanel extends FramedPanel
 
 	protected void create() {
 		SimpleContainer container = new SimpleContainer();
-		container.setHeight(FIELDSHEIGHT);
+		container.setHeight(CONTAINERHEIGHT);
 
 		v = new VerticalLayoutContainer();
 		v.setScrollMode(ScrollMode.AUTO);
-		container.add(v);
+		v.setAdjustForScroll(false);
 
 		btnSave = new TextButton("Save");
 		btnSave.setIcon(ResourceBundle.INSTANCE.save());
@@ -164,6 +175,10 @@ public class EditRowPanel extends FramedPanel
 			}
 		});
 
+		if (columns.size() < 1) {
+			btnSave.disable();
+		}
+
 		btnClose = new TextButton("Close");
 		btnClose.setIcon(ResourceBundle.INSTANCE.close());
 		btnClose.setIconAlign(IconAlign.RIGHT);
@@ -183,12 +198,53 @@ public class EditRowPanel extends FramedPanel
 		flowButton.add(btnSave, new BoxLayoutData(new Margins(2, 4, 2, 4)));
 		flowButton.add(btnClose, new BoxLayoutData(new Margins(2, 4, 2, 4)));
 
-		ArrayList<FieldLabel> fields = generateFields();
-		for (FieldLabel fl : fields) {
-			v.add(fl, new VerticalLayoutData(1, -1, new Margins(1)));
+		rowsId = new ArrayList<String>();
+		if (editRow) {
+			for (RowRaw rowRaw : rowsRaw) {
+				FieldSet fieldSet = new FieldSet();
+				fieldSet.setCollapsible(false);
+				fieldSet.setItemId(rowRaw.getRowId());
+				fieldSet.setHeadingText(rowRaw.getRowId());
+				fieldSet.setWidth(FIELDSETWIDTH);
+				rowsId.add(rowRaw.getRowId());
 
+				VerticalLayoutContainer fieldSetLayout = new VerticalLayoutContainer();
+				// fieldSetLayout.setScrollMode(ScrollMode.AUTO);
+				fieldSet.add(fieldSetLayout, new MarginData(0));
+
+				ArrayList<FieldLabel> fields = generateFields(rowRaw);
+				for (FieldLabel fl : fields) {
+					fieldSetLayout.add(fl, new VerticalLayoutData(1, -1,
+							new Margins(0)));
+				}
+
+				v.add(fieldSet, new VerticalLayoutData(1, -1, new Margins(1)));
+
+			}
+
+		} else {
+			FieldSet fieldSet = new FieldSet();
+			fieldSet.setCollapsible(false);
+			fieldSet.setItemId(ITEM_CREATE_ROW);
+			fieldSet.setHeadingText("New");
+			fieldSet.setWidth(FIELDSETWIDTH);
+			rowsId.add(ITEM_CREATE_ROW);
+
+			VerticalLayoutContainer fieldSetLayout = new VerticalLayoutContainer();
+			// fieldSetLayout.setScrollMode(ScrollMode.AUTO);
+			fieldSet.add(fieldSetLayout);
+
+			ArrayList<FieldLabel> fields = generateFields(null);
+			for (FieldLabel fl : fields) {
+				fieldSetLayout.add(fl, new VerticalLayoutData(1, -1,
+						new Margins(0)));
+			}
+			
+
+			v.add(fieldSet, new VerticalLayoutData(1, -1, new Margins(1)));
 		}
-
+		
+		container.add(v, new MarginData(0));
 		container.forceLayout();
 
 		VerticalLayoutContainer vPanel = new VerticalLayoutContainer();
@@ -207,10 +263,16 @@ public class EditRowPanel extends FramedPanel
 							eventBus.fireEvent(new SessionExpiredEvent(
 									SessionExpiredType.EXPIREDONSERVER));
 						} else {
-							Log.error("load columns failure:"
-									+ caught.getLocalizedMessage());
-							UtilsGXT3.alert("Error retrieving columns",
-									"Error retrieving columns");
+							if (caught instanceof TDGWTIsLockedException) {
+								Log.error(caught.getLocalizedMessage());
+								UtilsGXT3.alert("Error Locked",
+										caught.getLocalizedMessage());
+							} else {
+								Log.error("load columns failure:"
+										+ caught.getLocalizedMessage());
+								UtilsGXT3.alert("Error retrieving columns",
+										"Error retrieving columns");
+							}
 						}
 					}
 
@@ -223,9 +285,9 @@ public class EditRowPanel extends FramedPanel
 
 	}
 
-	protected ArrayList<FieldLabel> generateFields() {
+	protected ArrayList<FieldLabel> generateFields(RowRaw rowRaw) {
 		ArrayList<FieldLabel> fields = new ArrayList<FieldLabel>();
-		
+
 		for (ColumnData col : columns) {
 			String label = new String();
 			if (col != null && col.getLabel() != null) {
@@ -241,8 +303,8 @@ public class EditRowPanel extends FramedPanel
 						|| col.getTypeCode().compareTo(
 								ColumnTypeCode.TIMEDIMENSION.toString()) == 0) {
 
-					FieldLabel dimensionLabel = retrieveDimensionLabel(col,
-							label);
+					FieldLabel dimensionLabel = retrieveDimensionLabel(rowRaw,
+							col, label);
 					dimensionLabel.setLabelWidth(LABELSIZE);
 					fields.add(dimensionLabel);
 				} else {
@@ -325,6 +387,9 @@ public class EditRowPanel extends FramedPanel
 								if (col.getDataTypeName().compareTo(
 										ColumnDataType.Geometry.toString()) == 0) {
 									TextField geometry = new TextField();
+									geometry.addValidator(new RegExValidator(
+											GEOMETRY_REGEXPR,
+											"Geometry Type not valid"));
 									String value = "";
 									if (editRow) {
 										value = rowRaw.getMap().get(
@@ -408,9 +473,9 @@ public class EditRowPanel extends FramedPanel
 
 	}
 
-	protected FieldLabel retrieveDimensionLabel(final ColumnData col,
-			String label) {
-		Log.debug("retriveDimensionLabel on:"+col);
+	protected FieldLabel retrieveDimensionLabel(RowRaw rowRaw,
+			final ColumnData col, String label) {
+		Log.debug("retriveDimensionLabel on:" + col);
 		// comboDimension
 		DimensionRowsProperties propsDimension = GWT
 				.create(DimensionRowsProperties.class);
@@ -443,9 +508,8 @@ public class EditRowPanel extends FramedPanel
 			}
 		};
 
-		
 		String viewColumn = col.getRelationship().getTargetColumnId();
-		
+
 		String valueOnViewColumn = null;
 		String rowId = null;
 		if (editRow) {
@@ -488,135 +552,164 @@ public class EditRowPanel extends FramedPanel
 	protected void save() {
 		int i = 0;
 		int lenght = v.getWidgetCount();
-		HashMap<String, String> maps = new HashMap<String, String>();
+
+		HashMap<String, HashMap<String, String>> rowsMaps = new HashMap<String, HashMap<String, String>>();
 
 		for (; i < lenght; i++) {
-			FieldLabel fieldLabel = (FieldLabel) v.getWidget(i);
-			String columnId = fieldLabel.getId();
-			ColumnData colCurrent = null;
-			for (ColumnData col : columns) {
-				if (col.getColumnId().compareTo(columnId) == 0) {
-					colCurrent = col;
-					break;
+			FieldSet fieldSet = (FieldSet) v.getWidget(i);
+			VerticalLayoutContainer fieldSetLayout = (VerticalLayoutContainer) fieldSet
+					.getWidget();
+			int j = 0;
+			int fieldSetLayoutLenght = fieldSetLayout.getWidgetCount();
+			HashMap<String, String> maps = new HashMap<String, String>();
+			for (; j < fieldSetLayoutLenght; j++) {
+				FieldLabel fieldLabel = (FieldLabel) fieldSetLayout
+						.getWidget(j);
+				String columnId = fieldLabel.getId();
+				ColumnData colCurrent = null;
+				for (ColumnData col : columns) {
+					if (col.getColumnId().compareTo(columnId) == 0) {
+						colCurrent = col;
+						break;
+					}
 				}
-			}
-			if (colCurrent == null) {
-				Log.debug("Current col is null");
-				btnSave.enable();
-				return;
-			}
-			if (colCurrent.getTypeCode().compareTo(
-					ColumnTypeCode.DIMENSION.toString()) == 0
-					|| colCurrent.getTypeCode().compareTo(
-							ColumnTypeCode.TIMEDIMENSION.toString()) == 0) {
-				@SuppressWarnings("unchecked")
-				ComboBox<DimensionRow> comboDimension = (ComboBox<DimensionRow>) fieldLabel
-						.getWidget();
-				DimensionRow dimRow = comboDimension.getValue();
-				if (dimRow == null
-						|| (dimRow != null && (dimRow.getRowId() == null || dimRow
-								.getRowId().isEmpty()))) {
-					UtilsGXT3.alert("Attentions", "Select a valid value for "
-							+ colCurrent.getLabel());
+				if (colCurrent == null) {
+					Log.debug("Current col is null");
 					btnSave.enable();
 					return;
 				}
-				maps.put(columnId, dimRow.getRowId());
-			} else {
-				if (colCurrent.getDataTypeName().compareTo(
-						ColumnDataType.Boolean.toString()) == 0) {
-					HorizontalPanel hpanel = (HorizontalPanel) fieldLabel
+				if (colCurrent.getTypeCode().compareTo(
+						ColumnTypeCode.DIMENSION.toString()) == 0
+						|| colCurrent.getTypeCode().compareTo(
+								ColumnTypeCode.TIMEDIMENSION.toString()) == 0) {
+					@SuppressWarnings("unchecked")
+					ComboBox<DimensionRow> comboDimension = (ComboBox<DimensionRow>) fieldLabel
 							.getWidget();
-					Radio radioTrue = (Radio) hpanel.getWidget(0);
-					maps.put(columnId, radioTrue.getValue().toString());
+					DimensionRow dimRow = comboDimension.getValue();
+					if (dimRow == null
+							|| (dimRow != null && (dimRow.getRowId() == null || dimRow
+									.getRowId().isEmpty()))) {
+						UtilsGXT3.alert(
+								"Attentions",
+								"Select a valid value for "
+										+ colCurrent.getLabel());
+						btnSave.enable();
+						return;
+					}
+					maps.put(columnId, dimRow.getRowId());
 				} else {
 					if (colCurrent.getDataTypeName().compareTo(
-							ColumnDataType.Date.toString()) == 0) {
-						DateField date = (DateField) fieldLabel.getWidget();
-						if (date == null || date.getCurrentValue() == null) {
-							maps.put(columnId, null);
-						} else {
-							Date d = date.getCurrentValue();
-							String dateS = sdf.format(d);
-							maps.put(columnId, dateS);
-						}
+							ColumnDataType.Boolean.toString()) == 0) {
+						HorizontalPanel hpanel = (HorizontalPanel) fieldLabel
+								.getWidget();
+						Radio radioTrue = (Radio) hpanel.getWidget(0);
+						maps.put(columnId, radioTrue.getValue().toString());
 					} else {
 						if (colCurrent.getDataTypeName().compareTo(
-								ColumnDataType.Text.toString()) == 0) {
-							TextField text = (TextField) fieldLabel.getWidget();
-							String val = text.getCurrentValue();
-							if (val == null) {
-								val = "";
+								ColumnDataType.Date.toString()) == 0) {
+							DateField date = (DateField) fieldLabel.getWidget();
+							if (date == null || date.getCurrentValue() == null) {
+								maps.put(columnId, null);
+							} else {
+								Date d = date.getCurrentValue();
+								String dateS = sdf.format(d);
+								maps.put(columnId, dateS);
 							}
-							maps.put(columnId, val);
 						} else {
 							if (colCurrent.getDataTypeName().compareTo(
-									ColumnDataType.Geometry.toString()) == 0) {
-								TextField geometry = (TextField) fieldLabel
+									ColumnDataType.Text.toString()) == 0) {
+								TextField text = (TextField) fieldLabel
 										.getWidget();
-								String val = geometry.getCurrentValue();
+								String val = text.getCurrentValue();
 								if (val == null) {
 									val = "";
 								}
 								maps.put(columnId, val);
 							} else {
 								if (colCurrent.getDataTypeName().compareTo(
-										ColumnDataType.Integer.toString()) == 0) {
-									TextField integ = (TextField) fieldLabel
+										ColumnDataType.Geometry.toString()) == 0) {
+									TextField geometry = (TextField) fieldLabel
 											.getWidget();
-									@SuppressWarnings("unused")
-									Integer intege;
-									try {
-										intege = new Integer(
-												integ.getCurrentValue());
-									} catch (NumberFormatException e) {
+									if (geometry.isValid()) {
+										String val = geometry.getCurrentValue();
+										if (val == null) {
+											val = "";
+										}
+										maps.put(columnId, val);
+									} else {
 										UtilsGXT3
 												.alert("Attentions",
-														colCurrent.getLabel()
-																+ " is no a valid Integer type");
+														"The value of "
+																+ colCurrent
+																		.getLabel()
+																+ " is not a valid text representation for geometry type ( e.g. POINT(34 56) or LINESTRING(65 34, 56.43 78.65)!");
 										btnSave.enable();
 										return;
 									}
-
-									maps.put(columnId, integ.getCurrentValue());
 								} else {
 									if (colCurrent.getDataTypeName().compareTo(
-											ColumnDataType.Numeric.toString()) == 0) {
-										TextField numeric = (TextField) fieldLabel
+											ColumnDataType.Integer.toString()) == 0) {
+										TextField integ = (TextField) fieldLabel
 												.getWidget();
 										@SuppressWarnings("unused")
-										Double d;
-										String val = numeric.getCurrentValue();
-										if (val == null) {
-											Log.debug("Attentions"
-													+ colCurrent.getLabel()
-													+ " is no a valid Numeric type");
-											UtilsGXT3
-													.alert("Attentions",
-															colCurrent
-																	.getLabel()
-																	+ " is no a valid Numeric type");
-											btnSave.enable();
-											return;
-										}
+										Integer intege;
 										try {
-											d = new Double(val);
+											intege = new Integer(
+													integ.getCurrentValue());
 										} catch (NumberFormatException e) {
-											Log.debug("Attentions"
-													+ colCurrent.getLabel()
-													+ " is no a valid Numeric type");
 											UtilsGXT3
 													.alert("Attentions",
 															colCurrent
 																	.getLabel()
-																	+ " is no a valid Numeric type");
+																	+ " is no a valid Integer type");
 											btnSave.enable();
 											return;
 										}
-										maps.put(columnId,
-												numeric.getCurrentValue());
-									} else {
 
+										maps.put(columnId,
+												integ.getCurrentValue());
+									} else {
+										if (colCurrent.getDataTypeName()
+												.compareTo(
+														ColumnDataType.Numeric
+																.toString()) == 0) {
+											TextField numeric = (TextField) fieldLabel
+													.getWidget();
+											@SuppressWarnings("unused")
+											Double d;
+											String val = numeric
+													.getCurrentValue();
+											if (val == null) {
+												Log.debug("Attentions"
+														+ colCurrent.getLabel()
+														+ " is no a valid Numeric type");
+												UtilsGXT3
+														.alert("Attentions",
+																colCurrent
+																		.getLabel()
+																		+ " is no a valid Numeric type");
+												btnSave.enable();
+												return;
+											}
+											try {
+												d = new Double(val);
+											} catch (NumberFormatException e) {
+												Log.debug("Attentions"
+														+ colCurrent.getLabel()
+														+ " is no a valid Numeric type");
+												UtilsGXT3
+														.alert("Attentions",
+																colCurrent
+																		.getLabel()
+																		+ " is no a valid Numeric type");
+												btnSave.enable();
+												return;
+											}
+											maps.put(columnId,
+													numeric.getCurrentValue());
+										} else {
+
+										}
 									}
 								}
 							}
@@ -624,22 +717,22 @@ public class EditRowPanel extends FramedPanel
 					}
 				}
 			}
+
+			rowsMaps.put(fieldSet.getItemId(), maps);
+
 		}
 
 		EditRowSession editRowSession;
 		if (editRow) {
-			editRowSession = new EditRowSession(trId, columns, maps,
-					rowRaw.getRowId());
+			editRowSession = new EditRowSession(trId, columns, rowsMaps, rowsId);
 		} else {
-			editRowSession = new EditRowSession(trId, columns, maps);
+			editRowSession = new EditRowSession(trId, columns, rowsMaps);
 		}
 
 		callEditRow(editRowSession);
 
 	}
 
-	
-
 	protected void callEditRow(EditRowSession editRowSession) {
 
 		TDGWTServiceAsync.INSTANCE.startEditRow(editRowSession,
@@ -651,8 +744,20 @@ public class EditRowPanel extends FramedPanel
 							eventBus.fireEvent(new SessionExpiredEvent(
 									SessionExpiredType.EXPIREDONSERVER));
 						} else {
-							UtilsGXT3.alert("Error",
-									"Error in operation invocation!");
+							if (caught instanceof TDGWTIsLockedException) {
+								Log.error(caught.getLocalizedMessage());
+								UtilsGXT3.alert("Error Locked",
+										caught.getLocalizedMessage());
+							} else {
+								if (caught instanceof TDGWTIsFinalException) {
+									Log.error(caught.getLocalizedMessage());
+									UtilsGXT3.alert("Error Final",
+											caught.getLocalizedMessage());
+								} else {
+									UtilsGXT3.alert("Error",
+											"Error in operation invocation!");
+								}
+							}
 						}
 					}
 
@@ -663,8 +768,7 @@ public class EditRowPanel extends FramedPanel
 				});
 
 	}
-	
-	
+
 	protected void close() {
 		if (parent != null) {
 			parent.close();
